function [x,t] = ei(f, df, T, N, x0)
% EI  : Euler Implicite
% Entrées :
%   f   : fonction du type f(t, x) représentant l'EDO dx/dt = f(t, x)
%   df  : dérivée partielle de f par rapport à x, nécessaire pour Newton
%   T   : temps final
%   N   : nombre de pas de temps
%   x0  : condition initiale x(0) = x0
% Sorties :
%   x   : vecteur des solutions approximatives aux temps t
%   t   : vecteur des temps

x(1) = x0;          % Initialisation avec la condition initiale
dt = T / N;         % Pas de temps

for n = 1:N
    t(n) = (n - 1) * dt;  % Temps courant

    % Construction de l'équation implicite : G(X) = X - x(n) - dt * f(t(n+1), X)
    G  = @(X) (X - x(n) - dt * f(t(n) + dt, X));      % Fonction à annuler
    dG = @(X) (1 - dt * df(t(n) + dt, X));            % Dérivée de G pour Newton

    % === Méthode de Newton-Raphson ===
    eps = 1e-6;        % Tolérance pour le critère d'arrêt
    kmax = 1e3;        % Nombre maximal d’itérations
    k = 0;             % Compteur d’itérations
    y = x(n);          % Initialisation de Newton avec la valeur précédente

    while (abs(G(y)) > eps) && (k < kmax)
        y = y - G(y) / dG(y);   % Itération de Newton
        k = k + 1;
    end

    if k < kmax
        x(n + 1) = y;           % Solution trouvée
    else
        error('STOP')           % Erreur : Newton n’a pas convergé
    end
end

t(N + 1) = T;    % Ajout du temps final dans le vecteur des temps

end
